angular.module('peruhop.services', ['ngCordova', 'peruhop.config'])
    // DB wrapper
    .factory('Utils', function ($q, DB_CONFIG, DB, RemoteInfo, $location) {
        var self = this;

        self.checkDb = function () {
            console.log(DB);
            var q = $q.defer();
            if (DB.db == null) {
                self.initializeDb().then(function () {
                    q.resolve();
                });
            }
            return q.promise;
        };

        self.initializeDb = function () {
            var deferred = $q.defer();
            var updateDb = true;
            var remotedata, remoteDbVersion;
            var DbVersion = window.localStorage.getItem(DB_CONFIG.variableName);

            if (DbVersion) {
                console.log("version");
                //check update
                if (RemoteInfo.verifyConnection()) {
                    RemoteInfo.getVersionDb().then(function (response) {
                        remoteDbVersion = response;
                        console.log(remoteDbVersion, DbVersion);
                        if (remoteDbVersion != DbVersion && remoteDbVersion != -1) {
                            RemoteInfo.getDb().then(function (response) {
                                remotedata = response;
                                DB.updateDb(remotedata);
                                deferred.resolve(true);
                                window.localStorage.setItem(DB_CONFIG.variableName, remoteDbVersion);
                            });
                            updateDb = true;
                        } else {
                            updateDb = false;
                            DB.initialize();
                            deferred.resolve(true);
                        }
                        //deferred.resolve(true);
                    });
                } else {
                    updateDb = false;
                }
            } else {
                console.log("noversion");
                //init update or local
                if (RemoteInfo.verifyConnection()) {
                    RemoteInfo.getVersionDb().then(function (response) {
                        remoteDbVersion = response;
                        console.log(remoteDbVersion, DbVersion);
                        RemoteInfo.getDb().then(function (response) {
                            remotedata = response;
                            DB.updateDb(remotedata);
                            deferred.resolve(true);
                            window.localStorage.setItem(DB_CONFIG.variableName, remoteDbVersion);
                        });
                        //deferred.resolve(true);
                    });

                } else {
                    //init empty database
                    updateDb = false;
                }
            }
            console.log("updateDb", updateDb);
            //if (!updateDb) {
            //  DB.initialize();
            //  deferred.resolve(true);
            //  $location.path('/');
            //}
            return deferred.promise;
        };

        return self;
    })
    .factory('RemoteInfo', function (DB_CONFIG, $http, $q, $cordovaNetwork) {
        var self = this;

        self.verifyConnection = function () {
            if (typeof navigator.connection == 'object') {
                //no connection to internet no sync process happen
                if (navigator.connection.type == 'none') {
                    console.log("sinconexion");
                    return false;
                } else {
                    console.log("conexion");
                    return true;
                }
            } else {
                if (ionic.Platform.platform() == 'macintel') {
                    return true;
                }
                return false;
            }
        };

        self.getVersionDb = function () {
            //var dfd = $q.defer()
            //
            //setTimeout(function () {
            //  dfd.resolve('hola');
            //}, 6000)
            //
            //return dfd.promise
            return $http({method: 'GET', url: DB_CONFIG.remoteUrl + 'version'})
                .then(function (response) {
                    return response.data.hash;
                }, function () {
                    return -1;
                });
        };

        self.getDb = function (fn) {
            return $http({method: 'GET', url: DB_CONFIG.remoteUrl + 'data'})
                .then(function (response) {
                    return response.data.db;
                }, function () {
                    return false;
                });
        };

        return self;
    })
    .factory('DB', function ($q, $timeout, DB_CONFIG) {
        var self = this;
        self.db = null;

        self.initialize = function (update) {

            console.log('DB.initialize', self);
            if (self.db != null) {
                return true;
            }

            if (ionic.Platform.platform() == 'macintel') {
                self.db = window.openDatabase(DB_CONFIG.name, '1.0', 'database', -1);
            } else {
                self.db = window.sqlitePlugin.openDatabase({name: DB_CONFIG.name});
            }
        };

        self.updateDb = function (db) {
            // Use self.db = window.sqlitePlugin.openDatabase({name: DB_CONFIG.name}); in production
            console.log('DB.updateDb');
            if (self.db != null) {
                return true;
            }
            if (ionic.Platform.platform() == 'macintel') {
                self.db = window.openDatabase(DB_CONFIG.name, '1.0', 'database', -1);
            } else {
                self.db = window.sqlitePlugin.openDatabase({name: DB_CONFIG.name});
            }

            angular.forEach(DB_CONFIG.tables, function (table) {
                var columns = [];
                var columnNames = [];
                var columnFields = [];

                angular.forEach(table.columns, function (column) {
                    columns.push(column.name + ' ' + column.type);
                    columnNames.push(column.name);
                    columnFields.push('?');
                });

                var query = 'DROP TABLE IF EXISTS ' + table.name + ';';
                self.query(query);

                query = 'CREATE TABLE IF NOT EXISTS ' + table.name + ' (' + columns.join(',') + ')';
                self.query(query);
                console.log('Table ' + table.name + ' initialized');

                angular.forEach(db[table.name], function (row) {
                    var records = [];
                    var recordData = [];
                    //columns.push(column.name + ' ' + column.type);
                    angular.forEach(table.columns, function (column) {
                        if (row[column.name] == undefined) {
                            row[column.name] = null;
                        }
                        recordData.push(row[column.name]);
                    });

                    var query = 'INSERT INTO ' + table.name + ' (' + columnNames.join(',') + ') VALUES (' + columnFields.join(',') + ')';
                    self.query(query, recordData);
                });
                console.log('Table ' + table.name + ' updated');
            });
        };

        self.query = function (query, bindings) {
            bindings = typeof bindings !== 'undefined' ? bindings : [];
            var deferred = $q.defer();

            self.db.transaction(function (transaction) {
                transaction.executeSql(query, bindings, function (transaction, result) {
                    deferred.resolve(result);
                }, function (transaction, error) {
                    deferred.reject(error);
                });
            });

            return deferred.promise;
        };

        self.fetchAll = function (result) {
            var output = [];

            for (var i = 0; i < result.rows.length; i++) {
                output.push(result.rows.item(i));
            }

            return output;
        };

        self.fetch = function (result) {
            return result.rows.item(0);
        };

        return self;
    })
    // Resource service example
    .factory('Items', function (DB) {
        var self = this;

        self.all = function () {
            return DB.query('SELECT * FROM items')
                .then(function (result) {
                    return DB.fetchAll(result);
                });
        };

        self.allByCat = function (id) {
            return DB.query('SELECT * FROM items WHERE category_id = ?', [id])
                .then(function (result) {
                    return DB.fetchAll(result);
                });
        };

        self.getById = function (id) {
            return DB.query('SELECT * FROM items WHERE id = ?', [id])
                .then(function (result) {
                    return DB.fetch(result);
                });
        };

        return self;
    })
    // Resource service example
    .factory('Categories', function (DB) {
        var self = this;

        self.all = function () {
            return DB.query('SELECT * FROM categories')
                .then(function (result) {
                    return DB.fetchAll(result);
                });
        };

        self.getById = function (id) {
            return DB.query('SELECT * FROM categories WHERE id = ?', [id])
                .then(function (result) {
                    return DB.fetch(result);
                });
        };

        return self;
    });